@@ -84,7 +84,7 @@ class DurationDistribution:
             # Generate missing elements
             local_sample = self._generate_raw_sample(size - len(sample))
             # Filter out negative and out of limits elements
-            local_sample = [element for element in local_sample if element > 0.0]
+            local_sample = [element for element in local_sample if element >= 0.0]
             if self.min is not None:
                 local_sample = [element for element in local_sample if element >= self.min]
             if self.max is not None:
@@ -94,10 +94,11 @@ class DurationDistribution:
             i += 1
         # Check if all elements got generated
         if len(sample) < size:
-            print("Warning! "
+            default_value = self._replace_out_of_bounds_value()
+            print(f"Warning when generating sample of distribution {self}. "
                   "Too many iterations generating durations out of the distribution limits! "
-                  "Setting default values!")
-            sample += [self._replace_out_of_bounds_value()] * (size - len(sample))
+                  f"Filling missing values with default ({default_value})!")
+            sample += [default_value] * (size - len(sample))
         # Return complete sample
         return sample
 
@@ -144,7 +145,7 @@ class DurationDistribution:
 
     def _replace_out_of_bounds_value(self):
         new_value = None
-        if self.mean is not None and self.mean > 0:
+        if self.mean is not None and self.mean >= 0.0:
             # Set to mean
             new_value = self.mean
         if self.min is not None and self.max is not None:
@@ -157,26 +158,6 @@ class DurationDistribution:
         # Return fixed value
         return new_value
 
-    def generate_one_value_with_boundaries(self) -> float:
-        """
-        Generate value following the duration distribution and
-        limit it by using min and max values
-        """
-        while True:
-            [val] = self.generate_sample(1)
-
-            # TODO: expon does not support boundaries at the moment
-            # should be fixed
-            if self.type in [DistributionType.FIXED, DistributionType.UNIFORM, DistributionType.EXPONENTIAL]:
-                # fixed and uniform distributions are not limited by min and max
-                # those limitations are already implied during value generation
-                break
-
-            if self.min <= val <= self.max:
-                break
-
-        return val
-
     def scale_distribution(self, alpha: float) -> "DurationDistribution":
         return DurationDistribution(
             name=self.type,
@@ -320,26 +301,48 @@ class DurationDistribution:
                 minimum=float(distr_params[2]["value"]),
                 maximum=float(distr_params[3]["value"]),
             )
-
         return distribution
 
+    def to_simple_function_call(self) -> str:
+        if self.type == DistributionType.FIXED:
+            return f"fix({self.mean})"
+        elif self.type == DistributionType.UNIFORM:
+            return f"uniform({self.min}, {self.max})"
+        elif self.type == DistributionType.NORMAL:
+            return f"norm({self.mean}, {self.std}, {self.min}, {self.max})"
+        elif self.type == DistributionType.EXPONENTIAL:
+            return f"expon({self.mean}, {self.min}, {self.max})"
+        elif self.type == DistributionType.LOG_NORMAL:
+            return f"lognorm({self.mean}, {self.var}, {self.min}, {self.max})"
+        elif self.type == DistributionType.GAMMA:
+            return f"gamma({self.mean}, {self.var}, {self.min}, {self.max})"
+        else:
+            raise ValueError(f"Unsupported distribution: {self.type}")
+
     def __str__(self):
         return "DurationDistribution(name: {}, mean: {}, var: {}, std: {}, min: {}, max: {})".format(
             self.type.value, self.mean, self.var, self.std, self.min, self.max
         )
 
 
-def get_best_fitting_distribution(data: list, filter_outliers: bool = False) -> DurationDistribution:
+def get_best_fitting_distribution(
+    data: list,
+    filter_outliers: bool = False,
+    outlier_threshold: float = 20.0,
+) -> DurationDistribution:
     """
     Discover the distribution (exponential, normal, uniform, log-normal, and gamma) that best fits the values in [data].
 
-    :param data:            Values to fit a distribution for.
+    :param data: Values to fit a distribution for.
     :param filter_outliers: If true, remove outliers from the sample.
+    :param outlier_threshold: Threshold to consider an observation an outlier. Increasing this outlier increases the
+                              flexibility of the detection method, i.e., an observation needs to be further from the
+                              mean to be considered as outlier.
 
     :return: the best fitting distribution.
     """
     # Filter outliers
-    filtered_data = remove_outliers(data) if filter_outliers else data
+    filtered_data = remove_outliers(data, outlier_threshold) if filter_outliers else data
     # Check for fixed value
     fix_value = _check_fix(filtered_data)
     if fix_value is not None:
@@ -360,8 +363,8 @@ def get_best_fitting_distribution(data: list, filter_outliers: bool = False) ->
         ]
         if mean != 0:
             dist_candidates += [DurationDistribution("lognorm", mean, var, std, d_min, d_max)]
-            if var != 0:
-                dist_candidates += [DurationDistribution("gamma", mean, var, std, d_min, d_max)]
+            # if var != 0:
+            #     dist_candidates += [DurationDistribution("gamma", mean, var, std, d_min, d_max)]
         # Search for the best one within the candidates
         best_distribution = None
         best_emd = sys.float_info.max
@@ -395,18 +398,26 @@ def _check_fix(data: list, delta=5):
     return value
 
 
-def get_observations_histogram(data: list, num_bins: int = 20, filter_outliers: bool = False) -> dict:
+def get_observations_histogram(
+    data: list,
+    num_bins: int = 20,
+    filter_outliers: bool = False,
+    outlier_threshold: float = 20.0,
+) -> dict:
     """
     Build a histogram with the values in [data], with [num_bins] bins. It builds the histogram, computes the CDF and the values of each
     bin of the CDF.
 
-    :param data:            Data to build the histogram.
-    :param num_bins:        Number of bins to use in the histogram.
+    :param data: Data to build the histogram.
+    :param num_bins: Number of bins to use in the histogram.
     :param filter_outliers: If true, remove outliers from the sample.
+    :param outlier_threshold: Threshold to consider an observation an outlier. Increasing this outlier increases the
+                              flexibility of the detection method, i.e., an observation needs to be further from the
+                              mean to be considered as outlier.
 
     :return: A dict with the histogram in Prosimos format, storing the CDF values and middle points.
     """
-    filtered_durations = remove_outliers(data) if filter_outliers else data
+    filtered_durations = remove_outliers(data, outlier_threshold) if filter_outliers else data
     bins = np.linspace(min(filtered_durations), max(filtered_durations), num_bins + 1)
     hist, _ = np.histogram(filtered_durations, bins=bins)
     cdf = np.cumsum(hist)
